import 'package:objectid/objectid.dart';
import 'package:oobium_datastore/src/persist/feature.dart';
import 'package:oobium_datastore/src/state.dart';

abstract class Converter {
  const Converter();
  StateValue decodePut(DataRecord record);
  StateKey decodeRemove(DataRecord record);
  DataRecord encodePut(StateValue value);
  DataRecord encodeRemove(StateValue value);
}

dynamic jsonValue(v) {
  if(v == null) return null;
  if(v is Map)       return v.map((k,v) => MapEntry('$k', recordValue(v)));
  if(v is Iterable)  return v.map((e) => recordValue(e));
  if(v is String)    return v;
  if(v is num)       return v;
  if(v is bool)      return v;
  if(v is Enum)      return v.name;
  if(v is ObjectId)  return v.hexString;
  if(v is DateTime)  return v.millisecondsSinceEpoch;
  try {
    return v.toJson();
  } catch(e) {
    return v.toString();
  }
}

dynamic recordValue(v) {
  if(v == null)      return null;
  if(v is Map)       return v.isNotEmpty ? v.map((k,v) => MapEntry('$k', recordValue(v))) : null;
  if(v is Iterable)  return v.isNotEmpty ? v.map((e) => recordValue(e)).toList() : null;
  if(v is String)    return v.isNotEmpty ? v : null;
  if(v is num)       return v;
  if(v is bool)      return v;
  if(v is Enum)      return v.name;
  if(v is ObjectId)  return v.hexString;
  if(v is DateTime)  return v.millisecondsSinceEpoch;
  try {
    return v.toJson();
  } catch(e) {
    return v.toString();
  }
}
