class InvalidTypeException implements Exception {

  final Type? actual;
  final Type expected;
  final String? _message;

  InvalidTypeException({
    required this.actual,
    required this.expected,
    String? message
  }) : _message = message;

  InvalidTypeException.value(value, {required Type type, String? message}) :
      actual = value?.runtimeType, expected = type, _message = message;

  String get message => _message ?? 'expected $expected, but was $actual';

  @override
  String toString() => '$runtimeType: $message';
}

class IdTypeException implements Exception {

  final String message;

  IdTypeException(Type type) :
    message = 'invalid type used for key id: $type';

  @override
  String toString() => '$runtimeType: $message';
}

class MissingBuilderError extends Error {

  final String message;

  MissingBuilderError(Type type) :
    message = 'no builder registered for $type';

  @override
  String toString() => '$runtimeType: $message';
}

class MissingConverterError extends Error {

  final String message;

  MissingConverterError(String type) :
    message = 'no converter registered for $type';

  @override
  String toString() => '$runtimeType: $message';
}
