import 'dart:async';

class Executor {

  Executor? _parent;
  Future<void>? _last;
  bool _canceled = false;
  bool get isCanceled => _canceled || (_parent?.isCanceled == true);
  bool get isNotCanceled => !isCanceled;

  Future<void> operator +(FutureOr Function(Executor e) op) => _add(op);
  Future<void> operator <<(Executor e) => addChild(e);

  Future<T> add<T>(FutureOr<T> Function(Executor e) op) => _add(op).then((result) => result as T);

  Future<void> addChild(Executor e) {
    final op = _add((_) => e.flush());
    e._parent = this;
    return op;
  }

  Future<void> flush() => _last ?? Future.value();

  Future<void> cancel()  {
    _canceled = true;
    return _last ?? Future.value();
  }

  Future _add(FutureOr Function(Executor e) op) async {
    assert(_parent == null, 'cannot add operations once attached to a parent');

    if(isCanceled) {
      return;
    }
    
    final prev = _last;
    final completer = Completer.sync();
    _last = completer.future;
    if(prev != null) {
      await prev;
    }

    try {
      if(isNotCanceled) {
        return await op(this);
      }
    } finally {
      if(identical(_last, completer.future)) {
        _last = null;
      }
      completer.complete();
    }
  }
}
