import 'package:oobium_datastore/src/convert.dart';
import 'package:oobium_datastore/src/persist/feature.dart';
import 'package:oobium_datastore/src/state.dart';

class ModelContext {

  final StateContext _context;
  final Converter _converter;
  ModelContext(this._context, this._converter);

  Future<void> open() => _context.open();
  Future<void> flush() => _context.flush();
  Future<void> close() => _context.close();
  Future<void> destroy() => _context.destroy();
  Future<void> reset() => _context.reset();

  int get length => _context.length;
  bool get isEmpty => _context.isEmpty;
  bool get isNotEmpty => _context.isNotEmpty;
  bool get isOpen => _context.isOpen;
  bool get isNotOpen => _context.isNotOpen;

  T feature<T extends StateFeature>() => _context.feature<T>();

  Stream<String> export() async* {
    for(final record in _context.stateValues.map(_converter.encodePut)) {
      yield record.toString();
    }
  }

  Future<void> import(Stream<String> data) => _context.reset(data
      .map(DataRecord.fromLine)
      .where((m) => m.isNotDelete)
      .map(_converter.decodePut)
  );

}