import 'dart:async';
import 'dart:html';
import 'dart:indexed_db';

import 'package:oobium_datastore/src/persist/data.dart';
import 'package:oobium_datastore/src/persist/data_.dart' as base;
import 'package:oobium_logging/oobium_logging.dart';

class DataPlatform implements base.DataPlatform {

  final Data data;
  final _log = Logger('oobium_datastore').child(DataPlatform);
  DataPlatform(this.data);

  Database? idb;

  Future<void> open() async {
    assert(window.indexedDB != null);
    idb = await window.indexedDB!.open('${data.path}/${data.version}',
      version: data.version,
      onBlocked: onBlocked,
      onUpgradeNeeded: onUpgradeNeeded,
    );
  }

  dynamic connect() {
    return idb;
  }

  Future<void> close() async {
    idb = null;
  }

  Future<void> destroy() async {
    await window.indexedDB?.deleteDatabase(data.path);
  }

  void onBlocked(Event event) {
    _log.error('onBlocked');
  }

  void onUpgradeNeeded(VersionChangeEvent event) async {
    final oldVersion = event.oldVersion ?? 0;
    if(oldVersion < 1) {
      final upgradeDb = event.target.result as Database;
      if(upgradeDb.objectStoreNames?.contains('repo') != true) {
        upgradeDb.createObjectStore('repo');
      }
      if(upgradeDb.objectStoreNames?.contains('sync') != true) {
        upgradeDb.createObjectStore('sync');
      }
    }
    final newVersion = event.newVersion ?? ((oldVersion == 0) ? 1 : oldVersion);
    if(newVersion != oldVersion) {
      final oldData = (oldVersion > 0) ? (await Data(data.path, oldVersion)) : null;
      final updated = (await data.onUpgrade?.call(DataUpgradeEvent(oldVersion, oldData, newVersion, data))) ?? true;
      if(updated) {
        await oldData?.destroy();
      }
    }
  }
}
