import 'dart:async';
import 'dart:io';

import 'package:oobium_datastore/src/persist/data.dart';
import 'package:oobium_datastore/src/persist/data_.dart' as base;
import 'package:oobium_logging/oobium_logging.dart';

class DataPlatform implements  base.DataPlatform {

  final Data data;
  final _log = Logger('oobium_datastore').child(DataPlatform);
  DataPlatform(this.data);

  Directory? dir;

  Future<void> open() async {
    final vf = File('${data.path}/version');
    final oldVersion = (await vf.exists()) ? int.parse(await vf.readAsString()) : 0;
    final newVersion = data.version;
    dir = _dir;
    if(!await dir!.exists()) {
      await dir!.create(recursive: true);
    }
    if(newVersion != oldVersion) {
      final oldData = (oldVersion > 0) ? (await Data(data.path, oldVersion).open()) : null;
      final upgraded = await data.onUpgrade?.call(DataUpgradeEvent(oldVersion, oldData, newVersion, data));
      if(upgraded == true) {
        await vf.writeAsString('$newVersion');
        await oldData?.destroy();
      }
    }
  }

  dynamic connect() {
    return dir!.path;
  }

  Future<void> close() async {
    dir = null;
  }

  Future<void> destroy() async {
    final dir = _dir;
    if(await dir.exists()) {
      await dir.delete(recursive: true).catchError((_) {
        _log.error('destroy failed');
      });
    }
  }

  Directory get _dir => Directory('${data.path}/${data.version}');
}
