import 'dart:async';

import 'package:oobium_datastore/src/persist/data.dart';
import 'package:oobium_datastore/src/persist/feature.dart';
import 'package:oobium_datastore/src/persist/repo.dart';

class DataWorker {
  final String path;
  final int version;
  final OnUpgrade? _onUpgrade;
  late Data _data;
  late Repo _repo;
  DataWorker(this.path, this.version, [this._onUpgrade]);

  bool _open = false;
  bool get isOpen => _open;
  bool get isNotOpen => !isOpen;

  Future<DataWorker> open() async {
    assert(isNotOpen);
    _open = true;
    _data = await Data(path, version, (event) async {
      final onUpgrade = _onUpgrade;
      if(onUpgrade != null) {
        final oldRepo = (event.oldData != null) ? (await Repo(event.oldData!).open()) : null;
        final stream = onUpgrade(UpgradeEvent(event.oldVersion, event.newVersion, oldRepo?.get() ?? Stream<DataRecord>.empty()));
        final newRepo = await Repo(event.newData).open();
        newRepo.put(stream);
        await newRepo.flush();
        await newRepo.close();
        await oldRepo?.close();
      }
      return true;
    }).open();
    _repo = await Repo(_data).open();
    return this;
  }

  Future<void> flush() async {
    assert(isOpen);
    await _repo.flush();
  }

  Future<void> close() async {
    assert(isOpen);
    await _repo.flush();
    await _data.close();
  }

  Future<void> destroy() async {
    assert(isOpen);
    await _repo.close();
    await _data.destroy();
  }

  Stream<DataRecord> getData() {
    assert(isOpen);
    return _repo.get();
  }

  Future<void> putData({Iterable<DataRecord>? update, Iterable<DataRecord>? reset}) async {
    assert(isOpen);
    if(reset != null) {
      await _repo.reset(reset);
    }
    if(update != null) {
      await _repo.putAll(update);
    }
  }
}