import 'dart:io';

import 'package:args/command_runner.dart';
import 'models_test.schema.g.dart';

mixin _DataMixin {
  Future<ModelsTestData> Function()? _ds;
  Future<ModelsTestData> get modelsTestData =>
      _ds?.call() ?? ((this as Command).parent as _DataMixin).modelsTestData;
}

class ModelsTestDataCommand extends Command with _DataMixin {
  @override
  String get name => 'models_test_data';
  @override
  String get description => 'models test data commands';
  ModelsTestDataCommand([Future<ModelsTestData> Function()? ds]) {
    _ds = ds;
    addSubcommand(ModelsTestDataFetchCommand());
    addSubcommand(ModelsTestDataGetCommand());
  }
}

class ModelsTestDataFetchCommand extends Command with _DataMixin {
  @override
  String get name => 'fetch';
  @override
  String get description => 'models test data fetch commands';
  ModelsTestDataFetchCommand() {
    addSubcommand(ModelsTestDataFetchSettingsCommand());
    addSubcommand(ModelsTestDataFetchConfigCommand());
    addSubcommand(ModelsTestDataFetchImagesCommand());
    addSubcommand(ModelsTestDataFetchImageCommand());
    addSubcommand(ModelsTestDataFetchUsersCommand());
    addSubcommand(ModelsTestDataFetchUserCommand());
    addSubcommand(ModelsTestDataFetchPostsCommand());
    addSubcommand(ModelsTestDataFetchPostCommand());
    addSubcommand(ModelsTestDataFetchCommentsCommand());
    addSubcommand(ModelsTestDataFetchCommentCommand());
  }
}

class ModelsTestDataGetCommand extends Command with _DataMixin {
  @override
  String get name => 'get';
  @override
  String get description => 'models test data get commands';
  ModelsTestDataGetCommand() {
    addSubcommand(ModelsTestDataGetSettingsCommand());
    addSubcommand(ModelsTestDataGetConfigCommand());
    addSubcommand(ModelsTestDataGetAccountsCommand());
    addSubcommand(ModelsTestDataGetImagesCommand());
    addSubcommand(ModelsTestDataGetUsersCommand());
    addSubcommand(ModelsTestDataGetPostsCommand());
    addSubcommand(ModelsTestDataGetCommentsCommand());
  }
}

class ModelsTestDataFetchSettingsCommand extends Command with _DataMixin {
  @override
  String get name => 'settings';
  @override
  String get description => 'fetch Settings';
  @override
  Future<void> run() async {
    await (await modelsTestData).settings.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchConfigCommand extends Command with _DataMixin {
  @override
  String get name => 'config';
  @override
  String get description => 'fetch Config';
  @override
  Future<void> run() async {
    await (await modelsTestData).config.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchImagesCommand extends Command with _DataMixin {
  @override
  String get name => 'images';
  @override
  String get description => 'fetch Images';
  @override
  Future<void> run() async {
    await (await modelsTestData).images.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchImageCommand extends Command with _DataMixin {
  @override
  String get name => 'image';
  @override
  String get description => 'fetch Image';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = ImageKey.maybe(s);
      if (id != null) {
        await (await modelsTestData).images.fetch(id: id);
        stdout.writeln('done');
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataFetchUsersCommand extends Command with _DataMixin {
  @override
  String get name => 'users';
  @override
  String get description => 'fetch Users';
  @override
  Future<void> run() async {
    await (await modelsTestData).users.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchUserCommand extends Command with _DataMixin {
  @override
  String get name => 'user';
  @override
  String get description => 'fetch User';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = UserKey.maybe(s);
      if (id != null) {
        await (await modelsTestData).users.fetch(id: id);
        stdout.writeln('done');
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataFetchPostsCommand extends Command with _DataMixin {
  @override
  String get name => 'posts';
  @override
  String get description => 'fetch Posts';
  @override
  Future<void> run() async {
    await (await modelsTestData).posts.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchPostCommand extends Command with _DataMixin {
  @override
  String get name => 'post';
  @override
  String get description => 'fetch Post';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = PostKey.maybe(s);
      if (id != null) {
        await (await modelsTestData).posts.fetch(id: id);
        stdout.writeln('done');
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataFetchCommentsCommand extends Command with _DataMixin {
  @override
  String get name => 'comments';
  @override
  String get description => 'fetch Comments';
  @override
  Future<void> run() async {
    await (await modelsTestData).comments.fetch();
    stdout.writeln('done');
  }
}

class ModelsTestDataFetchCommentCommand extends Command with _DataMixin {
  @override
  String get name => 'comment';
  @override
  String get description => 'fetch Comment';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = CommentKey.maybe(s);
      if (id != null) {
        await (await modelsTestData).comments.fetch(id: id);
        stdout.writeln('done');
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataGetSettingsCommand extends Command with _DataMixin {
  @override
  String get name => 'settings';
  @override
  String get description => 'get Settings by id';
  @override
  Future<void> run() async {
    final settings = (await modelsTestData).settings;
    stdout.writeln(settings.toModel().toJson());
  }
}

class ModelsTestDataGetConfigCommand extends Command with _DataMixin {
  @override
  String get name => 'config';
  @override
  String get description => 'get Config by id';
  @override
  Future<void> run() async {
    final config = (await modelsTestData).config;
    stdout.writeln(config.toModel().toJson());
  }
}

class ModelsTestDataGetAccountsCommand extends Command with _DataMixin {
  @override
  String get name => 'accounts';
  @override
  String get description => 'get Accounts';
  @override
  Future<void> run() async {
    final accounts = (await modelsTestData).accounts;
    if (accounts.isEmpty) {
      stdout.writeln('no results');
    } else {
      for (final account in accounts) {
        stdout.writeln(account.toModel().toJson());
      }
    }
  }
}

class ModelsTestDataGetAccountCommand extends Command with _DataMixin {
  @override
  String get name => 'account';
  @override
  String get description => 'get Account by id';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = AccountKey.maybe(s);
      if (id != null) {
        final account = (await modelsTestData).accounts.getOrNull(id);
        if (account == null) {
          stdout.writeln('not found');
        } else {
          stdout.writeln(account.toModel().toJson());
        }
      } else {
        stdout.writeln('could not convert $s to ObjectId');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataGetImagesCommand extends Command with _DataMixin {
  @override
  String get name => 'images';
  @override
  String get description => 'get Images';
  @override
  Future<void> run() async {
    final images = (await modelsTestData).images;
    if (images.isEmpty) {
      stdout.writeln('no results');
    } else {
      for (final image in images) {
        stdout.writeln(image.toModel().toJson());
      }
    }
  }
}

class ModelsTestDataGetImageCommand extends Command with _DataMixin {
  @override
  String get name => 'image';
  @override
  String get description => 'get Image by id';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = ImageKey.maybe(s);
      if (id != null) {
        final image = (await modelsTestData).images.getOrNull(id);
        if (image == null) {
          stdout.writeln('not found');
        } else {
          stdout.writeln(image.toModel().toJson());
        }
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataGetUsersCommand extends Command with _DataMixin {
  @override
  String get name => 'users';
  @override
  String get description => 'get Users';
  @override
  Future<void> run() async {
    final users = (await modelsTestData).users;
    if (users.isEmpty) {
      stdout.writeln('no results');
    } else {
      for (final user in users) {
        stdout.writeln(user.toModel().toJson());
      }
    }
  }
}

class ModelsTestDataGetUserCommand extends Command with _DataMixin {
  @override
  String get name => 'user';
  @override
  String get description => 'get User by id';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = UserKey.maybe(s);
      if (id != null) {
        final user = (await modelsTestData).users.getOrNull(id);
        if (user == null) {
          stdout.writeln('not found');
        } else {
          stdout.writeln(user.toModel().toJson());
        }
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataGetPostsCommand extends Command with _DataMixin {
  @override
  String get name => 'posts';
  @override
  String get description => 'get Posts';
  @override
  Future<void> run() async {
    final posts = (await modelsTestData).posts;
    if (posts.isEmpty) {
      stdout.writeln('no results');
    } else {
      for (final post in posts) {
        stdout.writeln(post.toModel().toJson());
      }
    }
  }
}

class ModelsTestDataGetPostCommand extends Command with _DataMixin {
  @override
  String get name => 'post';
  @override
  String get description => 'get Post by id';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = PostKey.maybe(s);
      if (id != null) {
        final post = (await modelsTestData).posts.getOrNull(id);
        if (post == null) {
          stdout.writeln('not found');
        } else {
          stdout.writeln(post.toModel().toJson());
        }
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}

class ModelsTestDataGetCommentsCommand extends Command with _DataMixin {
  @override
  String get name => 'comments';
  @override
  String get description => 'get Comments';
  @override
  Future<void> run() async {
    final comments = (await modelsTestData).comments;
    if (comments.isEmpty) {
      stdout.writeln('no results');
    } else {
      for (final comment in comments) {
        stdout.writeln(comment.toModel().toJson());
      }
    }
  }
}

class ModelsTestDataGetCommentCommand extends Command with _DataMixin {
  @override
  String get name => 'comment';
  @override
  String get description => 'get Comment by id';
  @override
  Future<void> run() async {
    stdout.write('id: ');
    final s = stdin.readLineSync()?.trim();
    if (s != null) {
      final id = CommentKey.maybe(s);
      if (id != null) {
        final comment = (await modelsTestData).comments.getOrNull(id);
        if (comment == null) {
          stdout.writeln('not found');
        } else {
          stdout.writeln(comment.toModel().toJson());
        }
      } else {
        stdout.writeln('could not convert $s to int');
      }
    } else {
      stdout.writeln('canceled');
    }
  }
}
