part of 'models_test.schema.g.dart';

class _RestFeature extends BufferedStateFeature {
  late final ModelsTestData _modelContext;
  final SettingsRestAdapter settingsRestAdapter;
  final ConfigRestAdapter configRestAdapter;
  final AccountRestAdapter accountRestAdapter;
  final ImageRestAdapter imageRestAdapter;
  final UserRestAdapter userRestAdapter;
  final PostRestAdapter postRestAdapter;
  final CommentRestAdapter commentRestAdapter;
  _RestFeature({
    required this.settingsRestAdapter,
    required this.configRestAdapter,
    required this.accountRestAdapter,
    required this.imageRestAdapter,
    required this.userRestAdapter,
    required this.postRestAdapter,
    required this.commentRestAdapter,
  }) {
    settingsRestAdapter._feature = this;
    configRestAdapter._feature = this;
    accountRestAdapter._feature = this;
    imageRestAdapter._feature = this;
    userRestAdapter._feature = this;
    postRestAdapter._feature = this;
    commentRestAdapter._feature = this;
  }

  @override
  Future<void> onClose() {
    _expire();
    return super.onClose();
  }

  @override
  Future<void> onDestroy() {
    _expire();
    return super.onDestroy();
  }

  @override
  void onEvents(StateEvents events) {
    for (final event in events.created(this)) {
      if (event is StateEvent<SettingsSnap>) {
        _log.debug(() => 'created-Settings');
        q + (ex) => settingsRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<ConfigSnap>) {
        _log.debug(() => 'created-Config');
        q + (ex) => configRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<AccountSnap>) {
        _log.debug(() => 'created-Account');
        q + (ex) => accountRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<ImageSnap>) {
        _log.debug(() => 'created-Image');
        q + (ex) => imageRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<UserSnap>) {
        _log.debug(() => 'created-User');
        q + (ex) => userRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<PostSnap>) {
        _log.debug(() => 'created-Post');
        q + (ex) => postRestAdapter._created(ex, event.value);
      } else if (event is StateEvent<CommentSnap>) {
        _log.debug(() => 'created-Comment');
        q + (ex) => commentRestAdapter._created(ex, event.value);
      } else {
        _log.warn(() => 'created-UnknownType: $event');
      }
    }
    for (final event in events.updated(this)) {
      if (event is StateEvent<SettingsSnap>) {
        _log.debug(() => 'updated-Settings');
        q +
            (ex) => settingsRestAdapter._updated(
                ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<ConfigSnap>) {
        _log.debug(() => 'updated-Config');
        q +
            (ex) =>
                configRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<AccountSnap>) {
        _log.debug(() => 'updated-Account');
        q +
            (ex) =>
                accountRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<ImageSnap>) {
        _log.debug(() => 'updated-Image');
        q +
            (ex) =>
                imageRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<UserSnap>) {
        _log.debug(() => 'updated-User');
        q +
            (ex) =>
                userRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<PostSnap>) {
        _log.debug(() => 'updated-Post');
        q +
            (ex) =>
                postRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else if (event is StateEvent<CommentSnap>) {
        _log.debug(() => 'updated-Comment');
        q +
            (ex) =>
                commentRestAdapter._updated(ex, event.oldValue, event.newValue);
      } else {
        _log.warn(() => 'updated-UnknownType: $event');
      }
    }
    for (final event in events.removed(this)) {
      if (event is StateEvent<AccountSnap>) {
        _log.debug(() => 'removed-Account');
        q + (ex) => accountRestAdapter._removed(ex, event.value);
      } else if (event is StateEvent<ImageSnap>) {
        _log.debug(() => 'removed-Image');
        q + (ex) => imageRestAdapter._removed(ex, event.value);
      } else if (event is StateEvent<UserSnap>) {
        _log.debug(() => 'removed-User');
        q + (ex) => userRestAdapter._removed(ex, event.value);
      } else if (event is StateEvent<PostSnap>) {
        _log.debug(() => 'removed-Post');
        q + (ex) => postRestAdapter._removed(ex, event.value);
      } else if (event is StateEvent<CommentSnap>) {
        _log.debug(() => 'removed-Comment');
        q + (ex) => commentRestAdapter._removed(ex, event.value);
      } else {
        _log.warn(() => 'removed-UnknownType: $event');
      }
    }
  }

  void _expire() {
    settingsRestAdapter.expire();
    configRestAdapter.expire();
    imageRestAdapter.expire();
    userRestAdapter.expire();
    postRestAdapter.expire();
    commentRestAdapter.expire();
  }
}

abstract class _RestAdapter {
  late final _RestFeature _feature;
  ModelsTestData get _modelContext => _feature._modelContext;
}

class SettingsRestAdapter extends _RestAdapter {
  FutureOr<Map<String, dynamic>?> Function(ModelsTestData ds)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, Map<String, dynamic> data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, SettingsSnap settings)? _onCreated;
  FutureOr<void> Function(
      ModelsTestData ds, SettingsSnap old, SettingsSnap settings)? _onUpdated;

  SettingsRestAdapter(
      {FutureOr<Map<String, dynamic>?> Function(ModelsTestData ds)? onRequest,
      FutureOr<void> Function(ModelsTestData ds, Map<String, dynamic> data)?
          onResults,
      FutureOr<void> Function(ModelsTestData ds, SettingsSnap settings)?
          onCreated,
      FutureOr<void> Function(
              ModelsTestData ds, SettingsSnap old, SettingsSnap settings)?
          onUpdated})
      : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated;

  FutureOr<Map<String, dynamic>?> onRequest(ModelsTestData ds) =>
      _onRequest?.call(_modelContext);
  FutureOr<void> onResults(ModelsTestData ds, Map<String, dynamic> data) =>
      _onResults?.call(_modelContext, data);
  FutureOr<void> onCreated(ModelsTestData ds, SettingsSnap settings) =>
      _onCreated?.call(_modelContext, settings);
  FutureOr<void> onUpdated(
          ModelsTestData ds, SettingsSnap old, SettingsSnap settings) =>
      _onUpdated?.call(_modelContext, old, settings);

  FutureOr<void> _created(Executor ex, SettingsSnap settings) =>
      onCreated(_modelContext, settings);
  FutureOr<void> _updated(
          Executor ex, SettingsSnap old, SettingsSnap settings) =>
      onUpdated(_modelContext, old, settings);

  void expire() => _fetched = null;

  Future<void>? _fetched;
  Future<void> fetch(bool force) {
    if (force) {
      final current = _fetched;
      if (current != null) {
        _fetched = null;
        return current.then((_) => fetch(false));
      }
    }
    return (_fetched ??= _fetch());
  }

  Future<void> _fetch() async {
    _log.debug(() => 'fetching Settings');
    final data = await onRequest(_modelContext);
    if (data == null) {
      _log.debug(() => 'fetched Settings: not found');
    } else {
      await onResults(_modelContext, data);
      final settings = SettingsSnap.fromJson(data);
      _feature.put(settings);
      _log.debug(() => 'fetched $settings');
    }
  }
}

class ConfigRestAdapter extends _RestAdapter {
  FutureOr<Map<String, dynamic>?> Function(ModelsTestData ds)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, Map<String, dynamic> data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, ConfigSnap config)? _onCreated;
  FutureOr<void> Function(ModelsTestData ds, ConfigSnap old, ConfigSnap config)?
      _onUpdated;

  ConfigRestAdapter(
      {FutureOr<Map<String, dynamic>?> Function(ModelsTestData ds)? onRequest,
      FutureOr<void> Function(ModelsTestData ds, Map<String, dynamic> data)?
          onResults,
      FutureOr<void> Function(ModelsTestData ds, ConfigSnap config)? onCreated,
      FutureOr<void> Function(
              ModelsTestData ds, ConfigSnap old, ConfigSnap config)?
          onUpdated})
      : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated;

  FutureOr<Map<String, dynamic>?> onRequest(ModelsTestData ds) =>
      _onRequest?.call(_modelContext);
  FutureOr<void> onResults(ModelsTestData ds, Map<String, dynamic> data) =>
      _onResults?.call(_modelContext, data);
  FutureOr<void> onCreated(ModelsTestData ds, ConfigSnap config) =>
      _onCreated?.call(_modelContext, config);
  FutureOr<void> onUpdated(
          ModelsTestData ds, ConfigSnap old, ConfigSnap config) =>
      _onUpdated?.call(_modelContext, old, config);

  FutureOr<void> _created(Executor ex, ConfigSnap config) =>
      onCreated(_modelContext, config);
  FutureOr<void> _updated(Executor ex, ConfigSnap old, ConfigSnap config) =>
      onUpdated(_modelContext, old, config);

  void expire() => _fetched = null;

  Future<void>? _fetched;
  Future<void> fetch(bool force) {
    if (force) {
      final current = _fetched;
      if (current != null) {
        _fetched = null;
        return current.then((_) => fetch(false));
      }
    }
    return (_fetched ??= _fetch());
  }

  Future<void> _fetch() async {
    _log.debug(() => 'fetching Config');
    final data = await onRequest(_modelContext);
    if (data == null) {
      _log.debug(() => 'fetched Config: not found');
    } else {
      await onResults(_modelContext, data);
      final config = ConfigSnap.fromJson(data);
      _feature.put(config);
      _log.debug(() => 'fetched $config');
    }
  }
}

class AccountRestAdapter extends _RestAdapter {
  FutureOr<dynamic> Function(ModelsTestData ds, AccountQuery query)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, AccountQuery query, dynamic data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? _onCreated;
  FutureOr<void> Function(
      ModelsTestData ds, AccountSnap old, AccountSnap account)? _onUpdated;
  FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? _onRemoved;

  AccountRestAdapter({
    FutureOr<dynamic> Function(ModelsTestData ds, AccountQuery query)?
        onRequest,
    FutureOr<void> Function(
            ModelsTestData ds, AccountQuery query, dynamic data)?
        onResults,
    FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? onCreated,
    FutureOr<void> Function(
            ModelsTestData ds, AccountSnap old, AccountSnap account)?
        onUpdated,
    FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? onRemoved,
  })  : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated,
        _onRemoved = onRemoved;

  FutureOr<dynamic> onRequest(ModelsTestData ds, AccountQuery query) =>
      _onRequest?.call(_modelContext, query);
  FutureOr<void> onResults(
          ModelsTestData ds, AccountQuery query, dynamic data) =>
      _onResults?.call(_modelContext, query, data);
  FutureOr<void> onCreated(ModelsTestData ds, AccountSnap account) =>
      _onCreated?.call(_modelContext, account);
  FutureOr<void> onUpdated(
          ModelsTestData ds, AccountSnap old, AccountSnap account) =>
      _onUpdated?.call(_modelContext, old, account);
  FutureOr<void> onRemoved(ModelsTestData ds, AccountSnap account) =>
      _onRemoved?.call(_modelContext, account);

  FutureOr<void> _created(Executor ex, AccountSnap account) =>
      onCreated(_modelContext, account);
  FutureOr<void> _updated(Executor ex, AccountSnap old, AccountSnap account) =>
      onUpdated(_modelContext, old, account);
  FutureOr<void> _removed(Executor ex, AccountSnap account) =>
      onRemoved(_modelContext, account);

  void expire([AccountQuery? query]) {
    if (query == null || query.isAll) {
      _fetched.clear();
    } else {
      _fetched.remove(query.hashCode);
    }
  }

  final _fetched = <int, Future<void>>{};
  Future<void> fetch(AccountQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(AccountQuery query) async {
    _log.debug(() => 'fetching Account($query)');
    final data = await onRequest(_modelContext, query);
    if (data == null) {
      _log.debug(() => 'fetched Account: not found');
    } else if (data is Map<String, dynamic>) {
      await onResults(_modelContext, query, data);
      final account = AccountSnap.fromJson(data, withQuery: query);
      _feature.put(account);
      _log.debug(() => 'fetched $account');
    } else if (data is Iterable) {
      await onResults(_modelContext, query, data);
      final accounts = AccountSnap.fromJsonList(data, withQuery: query);
      _feature.set<AccountSnap>(accounts, where: query.test);
      _log.debug(() => 'fetched $accounts');
    } else {
      _log.debug(() => 'fetched Account: invalid result($data)');
    }
  }
}

class ImageRestAdapter extends _RestAdapter {
  FutureOr<dynamic> Function(ModelsTestData ds, ImageQuery query)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, ImageQuery query, dynamic data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? _onCreated;
  FutureOr<void> Function(ModelsTestData ds, ImageSnap old, ImageSnap image)?
      _onUpdated;
  FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? _onRemoved;

  ImageRestAdapter({
    FutureOr<dynamic> Function(ModelsTestData ds, ImageQuery query)? onRequest,
    FutureOr<void> Function(ModelsTestData ds, ImageQuery query, dynamic data)?
        onResults,
    FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? onCreated,
    FutureOr<void> Function(ModelsTestData ds, ImageSnap old, ImageSnap image)?
        onUpdated,
    FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? onRemoved,
  })  : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated,
        _onRemoved = onRemoved;

  FutureOr<dynamic> onRequest(ModelsTestData ds, ImageQuery query) =>
      _onRequest?.call(_modelContext, query);
  FutureOr<void> onResults(ModelsTestData ds, ImageQuery query, dynamic data) =>
      _onResults?.call(_modelContext, query, data);
  FutureOr<void> onCreated(ModelsTestData ds, ImageSnap image) =>
      _onCreated?.call(_modelContext, image);
  FutureOr<void> onUpdated(ModelsTestData ds, ImageSnap old, ImageSnap image) =>
      _onUpdated?.call(_modelContext, old, image);
  FutureOr<void> onRemoved(ModelsTestData ds, ImageSnap image) =>
      _onRemoved?.call(_modelContext, image);

  FutureOr<void> _created(Executor ex, ImageSnap image) =>
      onCreated(_modelContext, image);
  FutureOr<void> _updated(Executor ex, ImageSnap old, ImageSnap image) =>
      onUpdated(_modelContext, old, image);
  FutureOr<void> _removed(Executor ex, ImageSnap image) =>
      onRemoved(_modelContext, image);

  void expire([ImageQuery? query]) {
    if (query == null || query.isAll) {
      _fetched.clear();
    } else {
      _fetched.remove(query.hashCode);
    }
  }

  final _fetched = <int, Future<void>>{};
  Future<void> fetch(ImageQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(ImageQuery query) async {
    _log.debug(() => 'fetching Image($query)');
    final data = await onRequest(_modelContext, query);
    if (data == null) {
      _log.debug(() => 'fetched Image: not found');
    } else if (data is Map<String, dynamic>) {
      await onResults(_modelContext, query, data);
      final image = ImageSnap.fromJson(data, withQuery: query);
      _feature.put(image);
      _log.debug(() => 'fetched $image');
    } else if (data is Iterable) {
      await onResults(_modelContext, query, data);
      final images = ImageSnap.fromJsonList(data, withQuery: query);
      _feature.set<ImageSnap>(images, where: query.test);
      _log.debug(() => 'fetched $images');
    } else {
      _log.debug(() => 'fetched Image: invalid result($data)');
    }
  }
}

class UserRestAdapter extends _RestAdapter {
  FutureOr<dynamic> Function(ModelsTestData ds, UserQuery query)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, UserQuery query, dynamic data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, UserSnap user)? _onCreated;
  FutureOr<void> Function(ModelsTestData ds, UserSnap old, UserSnap user)?
      _onUpdated;
  FutureOr<void> Function(ModelsTestData ds, UserSnap user)? _onRemoved;

  UserRestAdapter({
    FutureOr<dynamic> Function(ModelsTestData ds, UserQuery query)? onRequest,
    FutureOr<void> Function(ModelsTestData ds, UserQuery query, dynamic data)?
        onResults,
    FutureOr<void> Function(ModelsTestData ds, UserSnap user)? onCreated,
    FutureOr<void> Function(ModelsTestData ds, UserSnap old, UserSnap user)?
        onUpdated,
    FutureOr<void> Function(ModelsTestData ds, UserSnap user)? onRemoved,
  })  : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated,
        _onRemoved = onRemoved;

  FutureOr<dynamic> onRequest(ModelsTestData ds, UserQuery query) =>
      _onRequest?.call(_modelContext, query);
  FutureOr<void> onResults(ModelsTestData ds, UserQuery query, dynamic data) =>
      _onResults?.call(_modelContext, query, data);
  FutureOr<void> onCreated(ModelsTestData ds, UserSnap user) =>
      _onCreated?.call(_modelContext, user);
  FutureOr<void> onUpdated(ModelsTestData ds, UserSnap old, UserSnap user) =>
      _onUpdated?.call(_modelContext, old, user);
  FutureOr<void> onRemoved(ModelsTestData ds, UserSnap user) =>
      _onRemoved?.call(_modelContext, user);

  FutureOr<void> _created(Executor ex, UserSnap user) =>
      onCreated(_modelContext, user);
  FutureOr<void> _updated(Executor ex, UserSnap old, UserSnap user) =>
      onUpdated(_modelContext, old, user);
  FutureOr<void> _removed(Executor ex, UserSnap user) =>
      onRemoved(_modelContext, user);

  void expire([UserQuery? query]) {
    if (query == null || query.isAll) {
      _fetched.clear();
    } else {
      _fetched.remove(query.hashCode);
    }
  }

  final _fetched = <int, Future<void>>{};
  Future<void> fetch(UserQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(UserQuery query) async {
    _log.debug(() => 'fetching User($query)');
    final data = await onRequest(_modelContext, query);
    if (data == null) {
      _log.debug(() => 'fetched User: not found');
    } else if (data is Map<String, dynamic>) {
      await onResults(_modelContext, query, data);
      final user = UserSnap.fromJson(data, withQuery: query);
      _feature.put(user);
      _log.debug(() => 'fetched $user');
    } else if (data is Iterable) {
      await onResults(_modelContext, query, data);
      final users = UserSnap.fromJsonList(data, withQuery: query);
      _feature.set<UserSnap>(users, where: query.test);
      _log.debug(() => 'fetched $users');
    } else {
      _log.debug(() => 'fetched User: invalid result($data)');
    }
  }
}

class PostRestAdapter extends _RestAdapter {
  FutureOr<dynamic> Function(ModelsTestData ds, PostQuery query)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, PostQuery query, dynamic data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, PostSnap post)? _onCreated;
  FutureOr<void> Function(ModelsTestData ds, PostSnap old, PostSnap post)?
      _onUpdated;
  FutureOr<void> Function(ModelsTestData ds, PostSnap post)? _onRemoved;

  PostRestAdapter({
    FutureOr<dynamic> Function(ModelsTestData ds, PostQuery query)? onRequest,
    FutureOr<void> Function(ModelsTestData ds, PostQuery query, dynamic data)?
        onResults,
    FutureOr<void> Function(ModelsTestData ds, PostSnap post)? onCreated,
    FutureOr<void> Function(ModelsTestData ds, PostSnap old, PostSnap post)?
        onUpdated,
    FutureOr<void> Function(ModelsTestData ds, PostSnap post)? onRemoved,
  })  : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated,
        _onRemoved = onRemoved;

  FutureOr<dynamic> onRequest(ModelsTestData ds, PostQuery query) =>
      _onRequest?.call(_modelContext, query);
  FutureOr<void> onResults(ModelsTestData ds, PostQuery query, dynamic data) =>
      _onResults?.call(_modelContext, query, data);
  FutureOr<void> onCreated(ModelsTestData ds, PostSnap post) =>
      _onCreated?.call(_modelContext, post);
  FutureOr<void> onUpdated(ModelsTestData ds, PostSnap old, PostSnap post) =>
      _onUpdated?.call(_modelContext, old, post);
  FutureOr<void> onRemoved(ModelsTestData ds, PostSnap post) =>
      _onRemoved?.call(_modelContext, post);

  FutureOr<void> _created(Executor ex, PostSnap post) =>
      onCreated(_modelContext, post);
  FutureOr<void> _updated(Executor ex, PostSnap old, PostSnap post) =>
      onUpdated(_modelContext, old, post);
  FutureOr<void> _removed(Executor ex, PostSnap post) =>
      onRemoved(_modelContext, post);

  void expire([PostQuery? query]) {
    if (query == null || query.isAll) {
      _fetched.clear();
    } else {
      _fetched.remove(query.hashCode);
    }
  }

  final _fetched = <int, Future<void>>{};
  Future<void> fetch(PostQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(PostQuery query) async {
    _log.debug(() => 'fetching Post($query)');
    final data = await onRequest(_modelContext, query);
    if (data == null) {
      _log.debug(() => 'fetched Post: not found');
    } else if (data is Map<String, dynamic>) {
      await onResults(_modelContext, query, data);
      final post = PostSnap.fromJson(data, withQuery: query);
      _feature.put(post);
      _log.debug(() => 'fetched $post');
    } else if (data is Iterable) {
      await onResults(_modelContext, query, data);
      final posts = PostSnap.fromJsonList(data, withQuery: query);
      _feature.set<PostSnap>(posts, where: query.test);
      _log.debug(() => 'fetched $posts');
    } else {
      _log.debug(() => 'fetched Post: invalid result($data)');
    }
  }
}

class CommentRestAdapter extends _RestAdapter {
  FutureOr<dynamic> Function(ModelsTestData ds, CommentQuery query)? _onRequest;
  FutureOr<void> Function(ModelsTestData ds, CommentQuery query, dynamic data)?
      _onResults;
  FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? _onCreated;
  FutureOr<void> Function(
      ModelsTestData ds, CommentSnap old, CommentSnap comment)? _onUpdated;
  FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? _onRemoved;

  CommentRestAdapter({
    FutureOr<dynamic> Function(ModelsTestData ds, CommentQuery query)?
        onRequest,
    FutureOr<void> Function(
            ModelsTestData ds, CommentQuery query, dynamic data)?
        onResults,
    FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? onCreated,
    FutureOr<void> Function(
            ModelsTestData ds, CommentSnap old, CommentSnap comment)?
        onUpdated,
    FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? onRemoved,
  })  : _onRequest = onRequest,
        _onResults = onResults,
        _onCreated = onCreated,
        _onUpdated = onUpdated,
        _onRemoved = onRemoved;

  FutureOr<dynamic> onRequest(ModelsTestData ds, CommentQuery query) =>
      _onRequest?.call(_modelContext, query);
  FutureOr<void> onResults(
          ModelsTestData ds, CommentQuery query, dynamic data) =>
      _onResults?.call(_modelContext, query, data);
  FutureOr<void> onCreated(ModelsTestData ds, CommentSnap comment) =>
      _onCreated?.call(_modelContext, comment);
  FutureOr<void> onUpdated(
          ModelsTestData ds, CommentSnap old, CommentSnap comment) =>
      _onUpdated?.call(_modelContext, old, comment);
  FutureOr<void> onRemoved(ModelsTestData ds, CommentSnap comment) =>
      _onRemoved?.call(_modelContext, comment);

  FutureOr<void> _created(Executor ex, CommentSnap comment) =>
      onCreated(_modelContext, comment);
  FutureOr<void> _updated(Executor ex, CommentSnap old, CommentSnap comment) =>
      onUpdated(_modelContext, old, comment);
  FutureOr<void> _removed(Executor ex, CommentSnap comment) =>
      onRemoved(_modelContext, comment);

  void expire([CommentQuery? query]) {
    if (query == null || query.isAll) {
      _fetched.clear();
    } else {
      _fetched.remove(query.hashCode);
    }
  }

  final _fetched = <int, Future<void>>{};
  Future<void> fetch(CommentQuery query, bool force) {
    if (force) {
      final current = _fetched.remove(query.hashCode);
      if (current != null) {
        return current.then((_) => fetch(query, false));
      }
    }
    return _fetched.putIfAbsent(query.hashCode, () => _fetch(query));
  }

  Future<void> _fetch(CommentQuery query) async {
    _log.debug(() => 'fetching Comment($query)');
    final data = await onRequest(_modelContext, query);
    if (data == null) {
      _log.debug(() => 'fetched Comment: not found');
    } else if (data is Map<String, dynamic>) {
      await onResults(_modelContext, query, data);
      final comment = CommentSnap.fromJson(data, withQuery: query);
      _feature.put(comment);
      _log.debug(() => 'fetched $comment');
    } else if (data is Iterable) {
      await onResults(_modelContext, query, data);
      final comments = CommentSnap.fromJsonList(data, withQuery: query);
      _feature.set<CommentSnap>(comments, where: query.test);
      _log.debug(() => 'fetched $comments');
    } else {
      _log.debug(() => 'fetched Comment: invalid result($data)');
    }
  }
}
