part of 'models_test.schema.g.dart';

class _TriggerFeature extends StateFeature {
  late final ModelsTestData _modelContext;
  final SettingsTriggers? settingsTriggers;
  final ConfigTriggers? configTriggers;
  final AccountTriggers? accountTriggers;
  final ImageTriggers? imageTriggers;
  final UserTriggers? userTriggers;
  final PostTriggers? postTriggers;
  final CommentTriggers? commentTriggers;
  _TriggerFeature({
    this.settingsTriggers,
    this.configTriggers,
    this.accountTriggers,
    this.imageTriggers,
    this.userTriggers,
    this.postTriggers,
    this.commentTriggers,
  });

  @override
  void onEvent(StateEvent e) {
    if (e.isCreate)
      _onCreate(e);
    else if (e.isUpdate)
      _onUpdate(e);
    else if (e.isRemove) _onRemove(e);
  }

  void _onCreate(StateEvent e) {
    if (e is StateEvent<SettingsSnap>) {
      _log.trace(() => 'created-Settings');
      settingsTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<ConfigSnap>) {
      _log.trace(() => 'created-Config');
      configTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<AccountSnap>) {
      _log.trace(() => 'created-Account');
      accountTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<ImageSnap>) {
      _log.trace(() => 'created-Image');
      imageTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<UserSnap>) {
      _log.trace(() => 'created-User');
      userTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<PostSnap>) {
      _log.trace(() => 'created-Post');
      postTriggers?.onCreate?.call(_modelContext, e.value);
    } else if (e is StateEvent<CommentSnap>) {
      _log.trace(() => 'created-Comment');
      commentTriggers?.onCreate?.call(_modelContext, e.value);
    }
  }

  void _onUpdate(StateEvent e) {
    if (e is StateEvent<SettingsSnap>) {
      _log.trace(() => 'updated-Settings');
      settingsTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<ConfigSnap>) {
      _log.trace(() => 'updated-Config');
      configTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<AccountSnap>) {
      _log.trace(() => 'updated-Account');
      accountTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<ImageSnap>) {
      _log.trace(() => 'updated-Image');
      imageTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<UserSnap>) {
      _log.trace(() => 'updated-User');
      userTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<PostSnap>) {
      _log.trace(() => 'updated-Post');
      postTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    } else if (e is StateEvent<CommentSnap>) {
      _log.trace(() => 'updated-Comment');
      commentTriggers?.onUpdate?.call(_modelContext, e.oldValue, e.newValue);
    }
  }

  void _onRemove(StateEvent e) {
    if (e is StateEvent<SettingsSnap>) {
      _log.trace(() => 'removed-Settings');
      for (final account
          in getAll<AccountSnap>((s) => s.settingsId == e.value.id)) {
        e.source.remove(account);
      }
    } else if (e is StateEvent<ConfigSnap>) {
      _log.trace(() => 'removed-Config');
    } else if (e is StateEvent<AccountSnap>) {
      _log.trace(() => 'removed-Account');
      accountTriggers?.onRemove?.call(_modelContext, e.value);
    } else if (e is StateEvent<ImageSnap>) {
      _log.trace(() => 'removed-Image');
      imageTriggers?.onRemove?.call(_modelContext, e.value);
    } else if (e is StateEvent<UserSnap>) {
      _log.trace(() => 'removed-User');
      for (final post in getAll<PostSnap>((s) => s.userId == e.value.id)) {
        e.source.remove(post);
      }
      for (final comment
          in getAll<CommentSnap>((s) => s.ownerId == e.value.id)) {
        e.source.remove(comment);
      }
      for (final comment
          in getAll<CommentSnap>((s) => s.auditorId == e.value.id)) {
        e.source.remove(comment);
      }
      userTriggers?.onRemove?.call(_modelContext, e.value);
    } else if (e is StateEvent<PostSnap>) {
      _log.trace(() => 'removed-Post');
      for (final comment
          in getAll<CommentSnap>((s) => s.postId == e.value.id)) {
        e.source.remove(comment);
      }
      postTriggers?.onRemove?.call(_modelContext, e.value);
    } else if (e is StateEvent<CommentSnap>) {
      _log.trace(() => 'removed-Comment');
      commentTriggers?.onRemove?.call(_modelContext, e.value);
    }
  }
}

class SettingsTriggers {
  FutureOr<void> Function(ModelsTestData ds, SettingsSnap settings)? onCreate;
  FutureOr<void> Function(
      ModelsTestData ds, SettingsSnap old, SettingsSnap settings)? onUpdate;

  SettingsTriggers({this.onCreate, this.onUpdate});
}

class ConfigTriggers {
  FutureOr<void> Function(ModelsTestData ds, ConfigSnap config)? onCreate;
  FutureOr<void> Function(ModelsTestData ds, ConfigSnap old, ConfigSnap config)?
      onUpdate;

  ConfigTriggers({this.onCreate, this.onUpdate});
}

class AccountTriggers {
  FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? onCreate;
  FutureOr<void> Function(
      ModelsTestData ds, AccountSnap old, AccountSnap account)? onUpdate;
  FutureOr<void> Function(ModelsTestData ds, AccountSnap account)? onRemove;

  AccountTriggers({this.onCreate, this.onUpdate, this.onRemove});
}

class ImageTriggers {
  FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? onCreate;
  FutureOr<void> Function(ModelsTestData ds, ImageSnap old, ImageSnap image)?
      onUpdate;
  FutureOr<void> Function(ModelsTestData ds, ImageSnap image)? onRemove;

  ImageTriggers({this.onCreate, this.onUpdate, this.onRemove});
}

class UserTriggers {
  FutureOr<void> Function(ModelsTestData ds, UserSnap user)? onCreate;
  FutureOr<void> Function(ModelsTestData ds, UserSnap old, UserSnap user)?
      onUpdate;
  FutureOr<void> Function(ModelsTestData ds, UserSnap user)? onRemove;

  UserTriggers({this.onCreate, this.onUpdate, this.onRemove});
}

class PostTriggers {
  FutureOr<void> Function(ModelsTestData ds, PostSnap post)? onCreate;
  FutureOr<void> Function(ModelsTestData ds, PostSnap old, PostSnap post)?
      onUpdate;
  FutureOr<void> Function(ModelsTestData ds, PostSnap post)? onRemove;

  PostTriggers({this.onCreate, this.onUpdate, this.onRemove});
}

class CommentTriggers {
  FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? onCreate;
  FutureOr<void> Function(
      ModelsTestData ds, CommentSnap old, CommentSnap comment)? onUpdate;
  FutureOr<void> Function(ModelsTestData ds, CommentSnap comment)? onRemove;

  CommentTriggers({this.onCreate, this.onUpdate, this.onRemove});
}
